@@ -40,10 +40,6 @@
    Reference: SVG working draft 3 March 2000, section 8.
 */
 
-#ifndef M_PI
-#define M_PI 3.14159265358979323846
-#endif  /*  M_PI  */
-
 typedef struct _RSVGParsePathCtx RSVGParsePathCtx;
 
 struct _RSVGParsePathCtx {
@@ -115,6 +111,14 @@ rsvg_path_arc (RSVGParsePathCtx *ctx,
   double th0, th1, th_arc;
   int i, n_segs;
 
+  /* Check that neither radius is zero, since its isn't either
+     geometrically or mathematically meaningful and will
+     cause divide by zero and subsequent NaNs.  We should
+     really do some ranged check ie -0.001 < x < 000.1 rather
+     can just a straight check again zero.
+  */
+  if ((rx == 0.0) || (ry == 0.0)) return;
+
   sin_th = sin (x_axis_rotation * (M_PI / 180.0));
   cos_th = cos (x_axis_rotation * (M_PI / 180.0));
   a00 = cos_th / rx;
@@ -194,15 +198,6 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
 {
   double x1, y1, x2, y2, x3, y3;
 
-#ifdef VERBOSE
-  int i;
-
-  g_print ("parse_path %c:", ctx->cmd);
-  for (i = 0; i < ctx->param; i++)
-    g_print (" %f", ctx->params[i]);
-  g_print (final ? ".\n" : "\n");
-#endif
-
   switch (ctx->cmd)
     {
     case 'm':
@@ -210,12 +205,8 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
       if (ctx->param == 2 || final)
 	{
 	  rsvg_parse_path_default_xy (ctx, 2);
-#ifdef VERBOSE
-	  g_print ("'m' moveto %g,%g\n",
-		   ctx->params[0], ctx->params[1]);
-#endif
 	  rsvg_bpath_def_moveto (ctx->bpath,
-					 ctx->params[0], ctx->params[1]);
+				 ctx->params[0], ctx->params[1]);
 	  ctx->cpx = ctx->rpx = ctx->params[0];
 	  ctx->cpy = ctx->rpy = ctx->params[1];
 	  ctx->param = 0;
@@ -226,12 +217,8 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
       if (ctx->param == 2 || final)
 	{
 	  rsvg_parse_path_default_xy (ctx, 2);
-#ifdef VERBOSE
-	  g_print ("'l' lineto %g,%g\n",
-		   ctx->params[0], ctx->params[1]);
-#endif
 	  rsvg_bpath_def_lineto (ctx->bpath,
-					 ctx->params[0], ctx->params[1]);
+				 ctx->params[0], ctx->params[1]);
 	  ctx->cpx = ctx->rpx = ctx->params[0];
 	  ctx->cpy = ctx->rpy = ctx->params[1];
 	  ctx->param = 0;
@@ -248,12 +235,8 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
 	  y2 = ctx->params[3];
 	  x3 = ctx->params[4];
 	  y3 = ctx->params[5];
-#ifdef VERBOSE
-	  g_print ("'c' curveto %g,%g %g,%g, %g,%g\n",
-		   x1, y1, x2, y2, x3, y3);
-#endif
 	  rsvg_bpath_def_curveto (ctx->bpath,
-					  x1, y1, x2, y2, x3, y3);
+				  x1, y1, x2, y2, x3, y3);
 	  ctx->rpx = x2;
 	  ctx->rpy = y2;
 	  ctx->cpx = x3;
@@ -272,12 +255,8 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
 	  y2 = ctx->params[1];
 	  x3 = ctx->params[2];
 	  y3 = ctx->params[3];
-#ifdef VERBOSE
-	  g_print ("'s' curveto %g,%g %g,%g, %g,%g\n",
-		   x1, y1, x2, y2, x3, y3);
-#endif
 	  rsvg_bpath_def_curveto (ctx->bpath,
-					  x1, y1, x2, y2, x3, y3);
+				  x1, y1, x2, y2, x3, y3);
 	  ctx->rpx = x2;
 	  ctx->rpy = y2;
 	  ctx->cpx = x3;
@@ -289,12 +268,8 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
       /* horizontal lineto */
       if (ctx->param == 1)
 	{
-#ifdef VERBOSE
-	  g_print ("'h' lineto %g,%g\n",
-		   ctx->params[0], ctx->cpy);
-#endif
 	  rsvg_bpath_def_lineto (ctx->bpath,
-					 ctx->params[0], ctx->cpy);
+				 ctx->params[0], ctx->cpy);
 	  ctx->cpx = ctx->rpx = ctx->params[0];
 	  ctx->param = 0;
 	}
@@ -303,12 +278,8 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
       /* vertical lineto */
       if (ctx->param == 1)
 	{
-#ifdef VERBOSE
-	  g_print ("'v' lineto %g,%g\n",
-		   ctx->cpx, ctx->params[0]);
-#endif
 	  rsvg_bpath_def_lineto (ctx->bpath,
-					 ctx->cpx, ctx->params[0]);
+				 ctx->cpx, ctx->params[0]);
 	  ctx->cpy = ctx->rpy = ctx->params[0];
 	  ctx->param = 0;
 	}
@@ -329,14 +300,10 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
 	  y3 = ctx->params[3];
 	  x2 = (x3 + 2 * ctx->params[0]) * (1.0 / 3.0);
 	  y2 = (y3 + 2 * ctx->params[1]) * (1.0 / 3.0);
-#ifdef VERBOSE
-	  g_print ("'q' curveto %g,%g %g,%g, %g,%g\n",
-		   x1, y1, x2, y2, x3, y3);
-#endif
 	  rsvg_bpath_def_curveto (ctx->bpath,
-					  x1, y1, x2, y2, x3, y3);
-	  ctx->rpx = x2;
-	  ctx->rpy = y2;
+				  x1, y1, x2, y2, x3, y3);
+	  ctx->rpx = ctx->params[0];
+	  ctx->rpy = ctx->params[1];
 	  ctx->cpx = x3;
 	  ctx->cpy = y3;
 	  ctx->param = 0;
@@ -357,12 +324,8 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
 	  y3 = ctx->params[1];
 	  x2 = (x3 + 2 * xc) * (1.0 / 3.0);
 	  y2 = (y3 + 2 * yc) * (1.0 / 3.0);
-#ifdef VERBOSE
-	  g_print ("'t' curveto %g,%g %g,%g, %g,%g\n",
-		   x1, y1, x2, y2, x3, y3);
-#endif
 	  rsvg_bpath_def_curveto (ctx->bpath,
-					  x1, y1, x2, y2, x3, y3);
+				  x1, y1, x2, y2, x3, y3);
 	  ctx->rpx = xc;
 	  ctx->rpy = yc;
 	  ctx->cpx = x3;
@@ -381,26 +344,18 @@ rsvg_parse_path_do_cmd (RSVGParsePathCtx *ctx, gboolean final)
 	      y3 = ctx->params[3];
 	      x2 = (x3 + 2 * ctx->params[0]) * (1.0 / 3.0);
 	      y2 = (y3 + 2 * ctx->params[1]) * (1.0 / 3.0);
-#ifdef VERBOSE
-	      g_print ("'t' curveto %g,%g %g,%g, %g,%g\n",
-		       x1, y1, x2, y2, x3, y3);
-#endif
 	      rsvg_bpath_def_curveto (ctx->bpath,
-					      x1, y1, x2, y2, x3, y3);
-	      ctx->rpx = x2;
-	      ctx->rpy = y2;
+				      x1, y1, x2, y2, x3, y3);
+              ctx->rpx = ctx->params[0];
+              ctx->rpy = ctx->params[1];
 	      ctx->cpx = x3;
 	      ctx->cpy = y3;
 	    }
 	  else
 	    {
 	      rsvg_parse_path_default_xy (ctx, 2);
-#ifdef VERBOSE
-	      g_print ("'t' lineto %g,%g\n",
-		       ctx->params[0], ctx->params[1]);
-#endif
 	      rsvg_bpath_def_lineto (ctx->bpath,
-					     ctx->params[0], ctx->params[1]);
+				     ctx->params[0], ctx->params[1]);
 	      ctx->cpx = ctx->rpx = ctx->params[0];
 	      ctx->cpy = ctx->rpy = ctx->params[1];
 	    }
@@ -518,7 +473,7 @@ rsvg_parse_path_data (RSVGParsePathCtx *ctx, const char *data)
 #else
 		  /* rule: even-numbered params are x-relative, odd-numbered
 		     are y-relative */
-		  if (ctx->param == 0 || (ctx->param % 2 ==0))
+		  if (ctx->param == 0 || (ctx->param % 2 == 0))
 		    val += ctx->cpx;
 		  else 
 		    val += ctx->cpy;
@@ -551,7 +506,7 @@ rsvg_parse_path_data (RSVGParsePathCtx *ctx, const char *data)
 	break;
       else if ((c == '+' || c == '-') && !exp_wait_sign)
 	{
-	  sign = c == '+' ? 1 : -1;;
+	  sign = c == '+' ? 1 : -1;
 	  val = 0;
 	  in_num = TRUE;
 	  in_frac = FALSE;
@@ -564,10 +519,10 @@ rsvg_parse_path_data (RSVGParsePathCtx *ctx, const char *data)
 	{
 	  if (ctx->param)
 	    rsvg_parse_path_do_cmd (ctx, TRUE);
-#ifdef VERBOSE
-	  g_print ("'z' closepath\n");
-#endif
 	  rsvg_bpath_def_closepath (ctx->bpath);
+
+	  ctx->cpx = ctx->rpx = ctx->bpath->bpath[ctx->bpath->n_bpath - 1].x3;
+	  ctx->cpy = ctx->rpy = ctx->bpath->bpath[ctx->bpath->n_bpath - 1].y3;
 	}
       else if (c >= 'A' && c <= 'Z' && c != 'E')
 	{
