@@ -168,6 +168,33 @@ rsvg_node_group_pack (RsvgNode *self, RsvgNode *child)
 	child->parent = self;
 }
 
+static gboolean
+rsvg_node_is_ancestor (RsvgNode * potential_ancestor,
+					   RsvgNode * potential_descendant)
+{
+	if(!potential_ancestor)
+		return FALSE;
+	else if(potential_descendant == potential_ancestor)
+		return TRUE;
+	else if(potential_ancestor->children) {
+		GPtrArray *children;
+		guint i, len;
+
+		children = potential_ancestor->children;
+		len = children->len;
+
+		for(i = 0; i < len; i++) {
+			RsvgNode * child;
+
+			child = (RsvgNode *)g_ptr_array_index (children, i);
+			if(rsvg_node_is_ancestor (child, potential_descendant))
+				return TRUE;
+		}
+	}
+		
+	return FALSE;
+}
+
 static void 
 rsvg_node_use_draw (RsvgNode * self, RsvgDrawingCtx *ctx, 
 							  int dominate)
@@ -187,8 +214,11 @@ rsvg_node_use_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 	child = use->link;
 
 	/* If it can find nothing to draw... draw nothing */
-	if (!use->link)
+	if (!child)
 		return;
+	else if(rsvg_node_is_ancestor (child, self)) /* or, if we're <use>'ing ourself */
+		return;
+
 	state = rsvg_state_current(ctx);	
 	if (strcmp(child->type->str, "symbol"))
 		{
@@ -325,11 +355,9 @@ rsvg_node_svg_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 static void
 rsvg_node_svg_set_atts (RsvgNode * self, RsvgHandle *ctx, RsvgPropertyBag *atts)
 {
-	const char * id, *value;
+	const char * id = NULL, *klazz = NULL, *value;
 	RsvgNodeSvg * svg = (RsvgNodeSvg *)self;
 
-	id = NULL;
-
 	if (rsvg_property_bag_size (atts))
 		{
 			if ((value = rsvg_property_bag_lookup (atts, "viewBox")))
@@ -345,11 +373,14 @@ rsvg_node_svg_set_atts (RsvgNode * self, RsvgHandle *ctx, RsvgPropertyBag *atts)
 				svg->x = _rsvg_css_parse_length (value);
 			if ((value = rsvg_property_bag_lookup (atts, "y")))
 				svg->y = _rsvg_css_parse_length (value);
+			if ((value = rsvg_property_bag_lookup (atts, "class")))
+				klazz = value;
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				{
 					id = value;
 					rsvg_defs_register_name (ctx->priv->defs, value, &svg->super);
 				}
+			rsvg_parse_style_attrs (ctx, self->state, "svg", klazz, id, atts);
 		}
 }
 
